/**
 * These classes are implementing the data store of the corpus_analysis
 * package. If we follow the idea of the Model View Controller Pattern these
 * classes combined in the Results class define the Model.
 */

// Results class bundleing the different data objects.
class Results {
  constructor() {
    this.data = new Data();
    this.metaData = new MetaData();
    this.fullResultsData = new Data();
    this.subResultsData = new Data();
    this.inspectResultsData = new Data();
    console.info('Initialized the Results object.');
  }

  // Reset all the data objects in the results class and thus emptying them.
  init() {
    this.data.init();
    this.metaData.init();
    this.fullResultsData.init();
    this.subResultsData.init();
    this.inspectResultsData.init();
    // Temporarly save match counts per text
    this.tmp_match_counts = {};
  }

}

/**
 * Class that defines the actual data objects holding the results data
 * requested by the client. Data can be the results of a query, full results
 * data for the export, sub results data for the export or inspect results data.
 * All kinds are structured the same way.
 */
class Data {
  /**
   * Sets empty object structure. Also usefull to delete old results.
   * MatchCount default is 0.
   */
  init(matchCount=0, type="results") {
    // List of all c with lc and rc CPOS.
    this.matches = [];
    /**
    * CPOS lookup object. CPOS are the key and value are infos about the CPOS
    * like lemma, ner, pos, text ID etc. CPOS from the matches correspond to
    * exactly one object in the cpos_lookup.
    */
    this.cpos_lookup = {};
    /**
    * Same like above but for text IDs. One CPOS object always has a text ID
    * referencing on text object in the text_lookup. Text ID is the key. Values
    * are author, publishing year etc.
    */
    this.text_lookup = {};
    this.match_count = matchCount;
    this.corpus_type = 'results';
    this.cpos_ranges = null;
    this.query = '';
  }

  /**
   * Function to add json data/object data to this data instance.
   * If no key is specified the entire data will be assigned to this data
   * instance.
   */
  addData(jsonData, key=null) {
    if (key !== null) {
      Object.assign(this[key], jsonData);
    } else if (key === null) {
      Object.assign(this, jsonData)
    }
  }

  // Get query as a string from the form Element.
  getQueryStr(queryFormElement) {
    // gets query
    let queryFormData;
    let queryStr;
    queryFormData = new FormData(queryFormElement);
    queryStr = queryFormData.get('query-form-query');
    this['query'] = queryStr;
  }

  // Function creates a unique and safe filename for the download.
  createDownloadFilename(suffix) {
    let today = new Date();
    let currentDate = `${today.getUTCFullYear()}` +
                      `-${(today.getUTCMonth() + 1)}` +
                      `-${today.getUTCDate()}`;
    let currentTime = `${today.getUTCHours()}h` +
                      `${today.getUTCMinutes()}m` +
                      `${today.getUTCSeconds()}s`;
    let safeFilename = this.query.replace(/[^a-z0-9_-]/gi, "_");
    let resultFilename = `UTC-${currentDate}_${currentTime}_${safeFilename}_${suffix}`;
    return resultFilename
  }
  /**
   * Function to download data as Blob created from string.
   * Should be private but that is not yet a feature of javascript 08.04.2020
   */
  download(downloadElement, dataStr, filename, type, filenameSlug) {
    filename += filenameSlug;
    let file = new Blob([dataStr], {type: type});
        var url = URL.createObjectURL(file);
        downloadElement.href = url;
        downloadElement.download = filename;
  }

  // Function to download the results as JSON.
  downloadJSONRessource(resultFilename, downloadData, downloadElement) {
    /**
     * Stringify JSON object for json download.
     * Use tabs to save some space.
     */
    let dataStr = JSON.stringify(downloadData, undefined, "\t");
    // Start actual download
    this.download(downloadElement, dataStr, resultFilename, "text/json", ".json")
  }

}

/**
 * Similar to the data class but just intended for meta data about the current
 * corpus the client is working with.
 */
class MetaData {
  // Sets empty object structure when no input is given.
  // if json object like input is given class fields are created from this
  init(json={}) {
    Object.assign(this, json);
  }
}

// Export the classes
export {
  Results,
  Data,
  MetaData
};